/*********************************************************************************
*                                                                                *
* MinPack (PokeMon Mini picture compression tool) by p0p for Team POKeme in 2004 *
*                     visit us @ http://pokeme.shizzle.it                        *
*                                                                                *
* File Structure of packed file (.mip) :                                         *
*                                                                                *
* 1st byte          = Cycle flag (flag byte) used to flag word pattern (2 bytes) *
* Any byte but flag = Byte will be copied without modification (not packed)      *
* Flag byte         = Next byte gives cycle count; next two are the fillbytes    *
*                                                                                *
*********************************************************************************/


#include "stdafx.h"

#define screensize 0x300

char *inputfile;
char *outputfile;
FILE *fi;
FILE *fo;


unsigned char infile[screensize];
unsigned char outfile[screensize];




void help()
{
	printf("\n* MinPack 1.0 by p0p for Team POKeMe *\n(Pokemon Mini picture compression tool)\nusage: minpack srcpic.lin targetpic.mip\n");
}


void error(char text[])
{
	fprintf(stderr, "\n**ERROR: %s\n",text);
}




int main(int argc, char *argv[])
{

	// handle parameters
	
	for (int a=1; a<argc; a++)
	{
		if (!inputfile)
			inputfile = argv[a];
		else if (!outputfile)
			outputfile = argv[a];
			else
			{
				error("too many filenames");
				return(-1);
			}	
	}


	if (!inputfile)
	{
		help();
		return 0;
	}

	if (!outputfile)
	{
		help();
		return -1;
	}



	// read linear bitmap file
	
	fi = fopen(inputfile, "rb");
	if (!fi)
	{
		error("input file not valid");
		return -1;
	}

	fseek(fi, 0, SEEK_END);

	if (int (ftell(fi))!=screensize)
	{
		error("input file size invalid");
		return -1;
	}

	fseek(fi, 0, SEEK_SET);
	fread (infile, 1, screensize, fi);
	fclose(fi);



	// determine flagbyte

	int found=0, flag=0, i=0, o=0, count=0, ab=0, bb=0;

	while ((flag<=0x100) && (!found))
	{
		found=1;
		for (int i=0; i<screensize; i++)
		{
			if (infile[i]==flag)
			{
				found=0;
				flag++;
			}
		}		
	}

	
	if (flag==0x100)
		error("no flagging possible on sourcefile");
	else
		outfile[o++]= char(flag);
		
		

	//main compression loop

		
	while (i<screensize)
	{
		ab=infile[i];
		bb=infile[i+1];

		if ( (ab==infile[i+2]) && (bb==infile[i+3]) && (i<(screensize-3)) && (count<0xff) && (flag!=0x100) )
		{
			count++;
			i+=2;
		}

		else { if ( (count) && (flag!=0x100) )
		{
			outfile[o++]=int(flag);
			outfile[o++]=int(count);
			outfile[o++]=int(ab);
			outfile[o++]=int(bb);
			count=0;
			i+=2;
		}
		else
			outfile[o++]=infile[i++];
		}
	}


	if (count)
	{
		outfile[o++]=int(flag);
		outfile[o++]=int(count);
		outfile[o++]=int(ab);
		outfile[o++]=int(bb);
	}



	// write outputfile

	if (o>=screensize)
		printf("WARNING: output file larger or equal input file!");

	fo = fopen(outputfile, "wb");
	
	if (!fo)
	{
		error("output file write failure");
		return -1;
	}

	fwrite(outfile, 1, o, fo);
	fclose(fo);

	printf("\nfile written (%d bytes, %f%%)\n",o,(double(o)/screensize)*100);


	return 0;
}




